/**
@author Cesar Vasquez
@version 1.1 14-06-04
*/
import java.applet.*;
import java.awt.event.*;
import java.net.*;
import javax.swing.*;
import java.io.*;
import java.lang.*;
import java.awt.*;
import javax.swing.filechooser.FileFilter;



/**
Applet encargada de el recibimiento de tareas via WEB.
*/
public class Proyecto extends JApplet
{
   private JTextArea textArea;

   /**
   Este metodo se crea cuando la Applet es cargada
   */
   public void init()
   {
      //se crean los botones y sus ActionListeners
      ActionListener sTarea = new SubirTarea();
      ActionListener vTarea = new VerTareas();
      JButton boton1 = new JButton("Subir Tarea");
      JButton boton2 = new JButton("Ver Tareas Subidas");
      boton1.addActionListener(sTarea);
      boton2.addActionListener(vTarea);
      getContentPane().setLayout(new BorderLayout());

      //se coloca una Etiqueta con el nombre del ramo
      String t = getParameter("ramo");
      JLabel titulo = new JLabel(
         "<html><h2><center>" + t + "</center></h2></html>");
      getContentPane().add(titulo, BorderLayout.NORTH);

      //se coloca en el centro un JTextArea para los mensajes
      textArea = new JTextArea("-- Reporte de Errores -- \n\n", 6, 40);
      textArea.setLineWrap(true);
      textArea.setFont(new Font("Times", Font.BOLD, 12));
      JScrollPane scrollPane = new JScrollPane(textArea);
      getContentPane().add(scrollPane, BorderLayout.CENTER);

      //se agregan los botones
      getContentPane().setComponentOrientation(
         ComponentOrientation.RIGHT_TO_LEFT);
      JPanel bp = new JPanel();
      bp.setLayout(new FlowLayout());
      bp.add(boton1);
      bp.add(boton2);
      getContentPane().add(bp, BorderLayout.SOUTH);
   }



   /**
   Action Listener para el boton de "Subir Tarea".
   Se encarga se subir el archivo una vez que el usuario
   lo elija.
   */
   private class SubirTarea implements ActionListener
   {
      /**
      Se ejecuta cuando el boton "Subir Tarea" es presionado
      @param event tiene informacion sobre el boton que es presionado.
      No sera utilizado por nosotros.
      */
      public void actionPerformed(ActionEvent event)
      {
         //se busca el archivo
         File tarea = this.BuscarArchivo();
         if(tarea!=null)
         {
            try
            {
               //se crea el Socket y los Streams
               String host = getParameter("host");
               Integer p = new Integer(getParameter("puerto"));
               int puerto = p.intValue();
               Socket clientSocket = new Socket(host,puerto);
               DataInputStream in = new DataInputStream(
                  new FileInputStream(tarea));
               DataOutputStream out = new DataOutputStream(
                  clientSocket.getOutputStream());
               String name = tarea.getName();
               //se envia tamao del archivo, el archivo, y su nombre
               long largo = tarea.length();
               out.writeLong(largo);
               for(int i=0; i<largo; i++)
                  out.write(in.read());
               out.writeBytes(name);

               //se cierran todos los canales
               in.close();
               out.close();
               clientSocket.close();

               //se imprime mensaje de exito
               String msg = new String(
                  "Tu tarea ha terminado de ser subida.\n"
                  + "Presiona el boton \"Ver Tareas Subidas\" para "
                  + "revisar que se encuentre en el servidor, y que el "
                  + "tamao sea el correcto.\n\n");
               textArea.append(msg);
            }
            catch(IOException e)
            {
               //si hay error se imprime mensaje de error
               e.printStackTrace();
               String msg = new String(
                  "Se ha producido un error al intentar subir tu tarea.\n"
                  + "Presiona el boton \"Ver Tareas Subidas\" para "
                  + "revisar que se encuentre en el servidor, y que el "
                  + "tamao sea el correcto.\n Si este no es el caso, contactese"
                  + " con su ayudante a la brevedad.\n\n");
               textArea.append(msg);
            }


         }
      }


      /**
      Este Metodo se encarga de crear y mostrar un FileChooser, para que el usuario
      eliga el archivo a enviar.
      */
      private File BuscarArchivo()
      {
         File tarea = null;
         //se crea un FileChooser
         JFileChooser fileChooser = new JFileChooser(".");
         //filtro para las extensiones
         ExtensionsFileFilter filter = new ExtensionsFileFilter();
         //se agregan las extensiones
         filter.addExtension(".tar");
         filter.addExtension(".zip");
         filter.setDescription("Archivos Comprimidos");
         //se elimina opcion "ver todos los archivos"
         fileChooser.setAcceptAllFileFilterUsed(false);
         fileChooser.setFileFilter(filter);
         int status = fileChooser.showOpenDialog(null);
         if (status == JFileChooser.APPROVE_OPTION)
         {
            File selectedFile = fileChooser.getSelectedFile();
            try
            {
               //se comprueba correcta extension
               String nombre = new String(selectedFile.getParent() + "\\"
                  + selectedFile.getName());
               //si la extension concuerda, se retorna el archivo
               if(nombre.endsWith(".zip") || nombre.endsWith(".tar"))
                  tarea = new File(nombre);
               else
               {
                  //si la extension no concuerda
                  String msg = new String(
                     "Recuerda que tu tarea debe estar comprimida en "
                     + "zip o tar. Revisa que est en el formato "
                     + "correcto antes de entregarla\n");
                  textArea.append(msg);
                  tarea = null;
               }

            }
            catch (Exception e)
            {
               //si hay algun error, se imprime mensaje, y se retorna null
               String msg = new String(
                  "Se ha producido un error al intentar enviar su archivo.\n "
                  + "Asegurese que el archivo exista o que la ruta sea la correcta. "
                  + "Si esta situacion se repite, contactese con su ayudante a la "
                  + "brevedad.\n\n");
               textArea.append(msg);
               e.printStackTrace();
            }
         }
         return tarea;
      }
   }





   /**
   Action Listener para el boton de "Ver Tareas Subidas".
   */
   private class VerTareas implements ActionListener
   {
      /**
      Este metodo se encarga de desplegar en una nueva ventana la lista
      con las tareas ya entregadas
      @param event tiene informacion sobre el boton que es presionado.
      No sera utilizado por nosotros.
      */
      public void actionPerformed(ActionEvent event)
      {
         try
         {
            //se despliega la pagina con la lista de tareas recibidas
            String pagina = getParameter("pagina");
            URL url = new URL (pagina + "/entregadas.txt");
            AppletContext context = getAppletContext();
            //se muestra en una nueva ventana
            context.showDocument(url, "_blank");

         }
         catch(MalformedURLException e)
         {
            String msg = new String(
               "Por alguna razon la pagina no puede ser desplegada. \n "
               + "Puede que el archivo no exista o que la ruta sea incorrecta. "
               + "Contactese con su ayudante para informar esta situacion\n\n");
            textArea.append(msg);
            e.printStackTrace();
         }
      }
   }
}
